{% extends "otree/Base.html" %}
{% load i18n otree_tags %}

{% block body_main %}
<div style="width:80%; margin:0 auto;">
<h2>{% block title %}
    {% trans 'Welcome' %}
{% endblock %}</h2>

<form method="get">
    {% if invalid_label %}
        <p class="error text-danger">Invalid label; try again.</p>
    {% endif %}
    <label for="participant_label">{% trans 'Please enter your participant label.' %}</label>
    <br/>
    <input type="text" id="participant_label" name="participant_label" class="form-control" />
    <br/>
    {% next_button %}
</form>
</div>
{% endblock %}